import createSerialNo from "~/mixin/createSerialNo"
import commonTemplate from "~/mixin/commonTemplate"
import commonDiary from "~/mixin/commonDiary"
import commonImage from "~/mixin/commonImage"
import commonCheck from "~/mixin/commonCheck"
import commonDeclare from "~/mixin/commonDeclare"

import Request from '~/utils/request'
import Utils from "~/utils";
import Api from "~/service/Api"
//import moment from "moment"

import {
  saveDraft,
  getDraft
} from "~/service/draft/draft"

export default {
  mixins: [createSerialNo, commonImage, commonDiary, commonTemplate, commonCheck, commonDeclare],
  data: function () {
    return {
      isReview: this.$route.path.indexOf('review') !== -1,
      process: '0', //0 经办 1 复核 2 经办 + 复核 3 经办更正
      taskFlag: '0', // 任务标识 1-由待办任务列表进入 2-由未完成任务 修改列表进入 3-未完成 详情 4- 由已完成交易列表进入 5-由草稿箱进入
      opNode: "0",
      version: 'v1', // 版本号
      oldModel: {},
      declareOldModel: {},
      urls: {
        startUrl: '', // 第一笔交易
        submitUrl: '', // 提交 通过 打回
        saveUrl: '', // 暂存
        loadUrl: '' // loaddata
      },
    }
  },
  created: function () {
    // todo: 获取 opNode

    this.urls.startUrl = `${this.version}/${this.declareParams.basePath}/start`
    this.urls.submitUrl = `${this.version}/${this.declareParams.basePath}/submit`
    this.urls.saveUrl = `${this.version}/${this.declareParams.basePath}/save`
    this.urls.loadUrl = `${this.version}/${this.declareParams.basePath}/getseqdata/`
    if (this.isReview) {
      this.process = this.$route.query.routeParams.process || '0'
      this.taskFlag = this.$route.query.routeParams.taskFlag || '0'
    } else {
      this.searchOrigin && this.searchOrigin()
    }
    if (this.$route.query.routeParams && this.$route.query.routeParams.txSriNo) {
      this.serialNo = this.$route.query.routeParams.txSriNo
    }
  },
  methods: {
    canEdit() {
      // 可编辑 可试算返回 true; 否则返回 false
      if (this.isReview && (this.process == '1' || this.taskFlag == '3' || this.taskFlag == '4')) {
        return false
      }
      return true
    },
    /**
     * 切换 tab 更新数据
     */
    async onTabClick() {
      // 复核阶段不重新试算
      if (this.canEdit()) {
        if (this.activeName == 'fee') {
          this.$refs.doc && this.updateDocFeeParamsList()
          this.$refs.fee && this.$refs.fee.calc()
        }

        if (this.activeName == 'doc' && this.$refs.doc) {
          if (this.getDocList) {
            let doclist = await this.getDocList() || []
            this.$refs.doc.calc(doclist)
          }
        }

        //当点击申报页签，且不为总行经办，则进行试算
        if (this.activeName.indexOf('declare') !== -1 && this.process !== '2' && this.process !== '4') {
          this.updateDeclare()
        }

      }

      if (this.activeName && this.activeName.toLowerCase().indexOf('mt') !== -1) {
        if (this.process == '0') {
          // 经办状态更新报文
          this.updateSwiftModel && this.updateSwiftModel()
        } else {
          // 复核阶段，已有报文数据，直接根据报文 id 取值

        }
      }
    },


    updateDocFeeParamsList() {
      // TODO: 获取全部面函列表，遍历产生报文费 面函费
      // 产生费用逻辑待讨论
      let docList = this.getWholeDocList()
      let templist = []
      let today = moment(new Date()).format('YYYYMMDD')
      if (docList.length > 0) {
        docList.forEach(item => {
          if (item.others && item.others.join(',').indexOf("01") != -1) {
            templist.push(
              Object.assign({
                ...this.feeOrigin
              }, {
                //赋值业务数据
                "feeStartDate": today,
                "feeEndDate": today,
                "feeUndertakSideCode": "0",
                //报文费用
                "feeCode": 'S0302024',
                "feeSourceCd": "T"
              })
            )
          }
          if (item.others && item.others.join(',').indexOf("02") != -1) {
            templist.push(
              Object.assign({
                ...this.feeOrigin
              }, {
                //赋值业务数据
                "feeStartDate": today,
                "feeEndDate": today,
                "feeUndertakSideCode": "0",
                //报文费用
                "feeCode": 'S0302025',
                "feeSourceCd": "T"
              })
            )
          }
        })
      }
      this.feeModel.docFeeParamsList = templist
    },

    getWholeDocList() {
      let list = []
      let doclist = this.getDocList && this.getDocList() || []
      list = this.$refs.doc && this.$refs.doc.getData && this.$refs.doc.getData(doclist) || []
      return list
    },

    /**
     * 获取数据
     */
    async loadData(params) {
      console.log('loaddata')
      // taskFlag 草稿箱从route取数据 其余从接口取数据
      if (this.taskFlag !== '5') {
        // 业务流水子表主键
        let busiTempInfoSriNo
        if (this.$route.path.indexOf('display') != -1 && params.busiTempInfoSriNo) {
          // 快照模式需要从参数中取交易编号
          busiTempInfoSriNo = params.busiTempInfoSriNo
        } else {
          busiTempInfoSriNo = this.$route.query.routeParams.busiTempInfoSriNo;
        }
        if (busiTempInfoSriNo) {
          const rtnmsg = await Api.pget(
            `${this.urls.loadUrl}${busiTempInfoSriNo}`
          );

          if (rtnmsg.code != "000000") {
            return this.$notify.error({
              title: "错误",
              message: "服务请求失败！"
            });
          }
          let busiTempData = JSON.parse(rtnmsg.data.busiTempData);
          let {
            head,
            body
          } = busiTempData;
          let {
            bizData
          } = body;
          if (head && head.reqSource !== '02' && head.reqSource !== '09') {
            Utils.copyValueFromVO(this.model, bizData);
          }
          // 请求头赋值 回显
          // Utils.copyValueFromVO(this.request.head, head);
          this.commonUpdateModel(body)
          this.getLoadData && this.getLoadData(bizData);
          // todo
          // this.opNode = body.opNode;

          // oldModel 赋值
          this.oldModel = JSON.parse(JSON.stringify(this.model));
          // 返回给复核界面
          let res = {
            tips: body.tips || [],
            changes: body.changes || [],
            remark: body.remark || ''
          }
          return res
        } else {
          this.$message.error('没有业务流水子表主键')
        }
      } else {
        this.loadDataFromDraft()
      }
      return {}
    },
    loadDataFromDraft() {
      if (this.$route.query.routeParams && this.$route.query.routeParams.txSriNo) {
        getDraft(this.$route.query.routeParams.txSriNo).then(res => {
          if (res && res.code == '000000' && res.data && res.data.busiCommitData) {
            let {
              bizData
            } = res.data.busiCommitData;
            Utils.copyValueFromVO(this.model, bizData);
            this.commonUpdateModel(res.data.busiCommitData)
          }
        })
      }
    },
    // 更新回显公共模块
    commonUpdateModel(body) {
      let {
        feeDataVo,
        accountDataVo,
        docLst,
        declare,
        diaLst
      } = body
      // 费用账务数据更新
      this.$refs.aml && this.serialNo && this.$refs.aml.getData(this.serialNo)
      this.$refs.fee && (feeDataVo || accountDataVo) && this.$refs.fee.updateModel(feeDataVo, accountDataVo)
      // this.$refs.images && imageLst && this.$refs.images.updateModel(imageLst)
      this.$refs.doc && docLst && this.$refs.doc.updateModel(docLst)
      this.$refs.diary && diaLst && this.$refs.diary.updateModel(diaLst)
      this.$nextTick(() => {
        this.updateDeclareModel(declare)
      })
    },

    /**
     * 提交数据
     */
    // 校验
    check(from = 'start') {
      let checkTasks = []
      // 自身表单校验，注意自身表单 ref='modelForm'
      if (this.$refs[this.formName]) {
        checkTasks.push(Utils.getFormPromise(this.$refs[this.formName]))
      }
      // 增加公共组件表单的校验
      if (this.checkCommonForm) {
        checkTasks.push(this.checkCommonForm())
      }
      // 额外的校验逻辑
      if (this.checkExceptForm) {
        checkTasks.push(this.checkExceptForm())
      }

      //费用账务校验
      // 更新面函报文list，必须在更新费用之前
      this.$refs.doc && this.updateDocFeeParamsList()
      this.$refs.fee && checkTasks.push(this.$refs.fee.check())

      if (this.$refs.fingrt && this.getForFingrt) {
        checkTasks.push(this.$refs.fingrt.check(this.getForFingrt()))
      }

      //总行经办在submit的check时做了试算，申报model做了增量替换，受理中心/总行复核不做trial
      if (this.model.declareMod && (this.model.declareMod.safeDeclWay !== "3" || this.model.declareMod.rcpmisDeclWay !== "2") && this.process !=='1') {
        this.model.declareMod && this.hasDeclare() && checkTasks.push(this.updateDeclare())
      }

      if (this.$refs.basic && this.$refs.basic.$refs.declare && this.$refs.basic.$refs.declare.$refs.declareModelForm) {
        checkTasks.push(Utils.getFormPromise(this.$refs.basic.$refs.declare.$refs.declareModelForm, '申报'))
      }

      if (this.$refs['declare_ort'] && this.$refs['declare_ort'].check) {
        checkTasks.push(this.$refs['declare_ort'].check('境外汇款申请书'))
      }

      if (this.$refs['declare_drt'] && this.$refs['declare_drt'].check) {
        checkTasks.push(this.$refs['declare_drt'].check('境内汇款申请书'))
      }

      if (this.$refs['declare_oin'] && this.$refs['declare_oin'].check) {
        checkTasks.push(this.$refs['declare_oin'].check('涉外收入申报单'))
      }

      if (this.$refs['declare_din'] && this.$refs['declare_din'].check) {
        checkTasks.push(this.$refs['declare_din'].check('境内收入申报单'))
      }

      if (this.$refs['declare_extGuarSign'] && this.$refs['declare_extGuarSign'].check) {
        checkTasks.push(this.$refs['declare_extGuarSign'].check('对外担保签约信息'))
      }

      if (this.$refs['declare_extGuarBal'] && this.$refs['declare_extGuarBal'].check) {
        checkTasks.push(this.$refs['declare_extGuarBal'].check('对外担保余额信息'))
      }

      if (this.$refs['declare_opy'] && this.$refs['declare_opy'].check) {
        checkTasks.push(this.$refs['declare_opy'].check('对外付款/承兑通知书'))
      }

      if (this.$refs['declare_pty'] && this.$refs['declare_pty'].check) {
        checkTasks.push(this.$refs['declare_pty'].check('境内付款/承兑通知书'))
      }

      return Promise.all(checkTasks).then(res => {
        // 所有校验都为 true时，校验通过
        if (from == 'save') {
          // 校验不需要 check
          return Promise.resolve(true)
        }
        console.log('check')
        console.log(res)
        let flag = res.every(item => {
          return !!item
        })
        if (flag) {
          this.$message.success('校验通过')
          return Promise.resolve(true)
        } else {
          this.scrollToCheckError && this.scrollToCheckError()
          return Promise.resolve(false)
        }
      }).catch(err => {
        return Promise.resolve(false)
      })
    },

    // 生成请求 head
    genRequestHead(step = undefined) {
      // 生成 request.head
      let head = {
        opNode: this.opNode, // 操作环节为：受理中心经办岗
        reqSource: "03",
        txInstNo: window.sessionStorage.instCode,
        oprInstNo: window.sessionStorage.instCode,
        oprTellerNo: window.sessionStorage.userId,
        createTellerNo: window.sessionStorage.userId,
        acceptCenterInstNo: window.sessionStorage.instCode,
        ...this.declareParams,
      }
      if (this.$route.query && this.$route.query.routeParams && this.$route.query.routeParams.amlTempData) {
        head.amlTempData = this.$route.query.routeParams.amlTempData
      }
      if (this.$route.query && this.$route.query.routeParams) {
        head.txSriNo = this.$route.query.routeParams.txSriNo || this.serialNo || ''
      }
      if (this.$route.query && this.$route.query.routeParams && this.$route.query.routeParams.taskId) {
        head.taskId = this.$route.query.routeParams.taskId
      }
      if (this.$route.query && this.$route.query.routeParams && this.$route.query.routeParams.taskFlag) {
        head.taskFlag = this.$route.query.routeParams.taskFlag
      }

      if (step == 2) {
        head.taskStatus = 1
      }
      if (step == 3) {
        head.taskStatus = 2
      }
      let list = this.getDocList && this.getDocList() || []
      head.isMessaged = false
      return head
    },
    // 生成公共请求 body
    async genCommonBody() {
      // 业务数据
      let commonBody = {}
      commonBody.bizData = Utils.flatObject(this.model)

      // fee 封装 params.body
      this.$refs.fee && this.$refs.fee.bodyWrapper(commonBody)
      this.$refs.images && this.imagesBodyWrapper && this.imagesBodyWrapper(commonBody)
      this.$refs.doc && this.$refs.doc.bodyWrapper(commonBody)
      this.$refs.diary && await this.$refs.diary.bodyWrapper(commonBody)
      this.model.declareMod && this.hasDeclare() && this.genDeclareData(commonBody)
      return commonBody
    },

    async submit(data, step = 2) {
      if (this.$refs.aml) {
        this.aml()
      }
      // step 1 提交 2 通过 3 退回
      let params = {
        head: this.genRequestHead(step),
        body: {
          ...await this.genCommonBody(),
        }
      }
      if (this.process == '0' && this.$refs.aml) {
        // 经办提交需要传反洗钱参数
        params.body.amlCheckVo = this.amlCheckModel
      }
      if (step == 2 && (this.process == '3' || this.process == '4')) {
        // 经办提交
        params.body.changes = Utils.compareChanges(this.oldModel, this.model)
      }
      if (step == 2 || step == 3) {
        // 通过 打回
        params.body.tips = (data && data.tips) || []
        params.body.remark = (data && data.remark) || ''
      }

      //总行经办需要比较新旧申报model ps：此时check流程已走完，走了trial
      if (this.process == '2' || this.process == '4') {
        if(this.declareParams.txCode === '040101' && this.model.declareMod.safeDeclWay !== "3" && this.model.declareMod.rcpmisDeclWay !== "2"){
          if (this.updateChangeDeclare() == true){
            this.assignDeclareModel(params);              
            //外管申报方式为随业务申报
            if (params.body.declare && params.body.declare.safeDeclWay == '1') {
              //此时params.body.declare数据为新model数据，不符合要求，需要旧数据
              params.body.declare.safeDeclWay = '2';           
            }                    
          }            
        }        
      }
      return Request.post(this.urls.submitUrl, params)
    },

    async start() {
      if (this.$refs.aml) {
        this.aml()
      }
      let params = {
        head: this.genRequestHead(),
        body: await this.genCommonBody()
      }
      // 生成交易流水号
      this.genRequestSerialNo(params)
      if (this.process == '0' && this.$refs.aml) {
        // 经办提交需要传反洗钱参数
        params.body.amlCheckVo = this.amlCheckModel
      }
      return Request.post(this.urls.startUrl, params)
    },

    save() {
      if (this.isReview && this.taskFlag !== '5') {
        return this.saveByBusi()
      } else {
        return this.saveByDraft()
      }
    },
    // 暂存进大字段
    saveByBusi() {
      let params = {
        head: this.genRequestHead(),
        body: this.genSaveBody()
      }
      return Request.post(this.urls.saveUrl, params)
    },
    // 暂存进草稿箱
    saveByDraft() {
      let params = {
        txSriNo: this.serialNo,
        txCode: this.declareParams.txCode,
        tsTellerNo: window.sessionStorage.userId,
        busiNo: this.declareParams.busiNo,
        busiCommitData: this.genSaveBody()
      }
      return saveDraft(params)
    },
    genSaveBody() {
      let body = {}
      body.bizData = Utils.flatObject(this.model)
      this.$refs.fee && this.$refs.fee.saveBodyWrapper(body)
      this.$refs.doc && this.$refs.doc.bodyWrapper(body)
      this.model.declareMod && this.hasDeclare() && this.genDeclareData(body)
      return body
    },

    // 需要实现的按钮方法
    handleStart() {
      this.beforeStart && this.beforeStart()
      this.check().then(res => {
        if (res) {
          this.start().then(res => {
            if (res && res.code == '000000') {
              this.$notify.success({
                title: "成功",
                message: "服务请求成功!"
              })
              this.exit()
            } else if (res.code == "200000" || res.code == "200001" || res.code == "200002") {
              this.$refs.aml && this.$refs.aml.setData(res.data)
            } else {
              this.$notify.error({
                title: "失败",
                message: "服务请求失败!"
              })
            }
          })
        }
      })
    },

    handleSubmit(data) {
      this.beforeSubmit && this.beforeSubmit()
      this.check().then(res => {
        if (res) {
          this.submit(data, 2).then(res => {
            if (res && res.code == '000000') {
              this.$notify.success({
                title: "成功",
                message: "服务请求成功!"
              })
              this.exit()
            } else if (res.code == "200000" || res.code == "200001" || res.code == "200002") {
              this.$refs.aml && this.$refs.aml.setData(res.data)
            } else {
              this.$notify.error({
                title: "失败",
                message: "服务请求失败!"
              })
            }
          })
        }
      })
    },

    handlePass(data) {
      this.check().then(res => {
        if (res) {
          this.submit(data, 2).then(res => {
            if (res && res.code == '000000') {
              this.$notify.success({
                title: "成功",
                message: "服务请求成功!"
              })
              this.exit()
            } else {
              this.$notify.error({
                title: "失败",
                message: "服务请求失败!"
              })
            }
          })
        }
      })
    },

    handleRefuse(data) {
      this.check().then(res => {
        if (res) {
          this.submit(data, 3).then(res => {
            if (res && res.code == '000000') {
              this.$notify.success({
                title: "成功",
                message: "服务请求成功!"
              })
              this.exit()
            } else {
              this.$notify.error({
                title: "失败",
                message: "服务请求失败!"
              })
            }
          })
        }
      })
    },

    handleCheck() {
      this.check()
    },

    handleSave() {
      if (this.declareParams.busiNo) {
        this.$confirm('确认暂存？', '', {
          confirmButtonText: '确认',
          cancelButtonText: '取消',
          type: 'warning'
        }).then(() => {
          this.beforeSave && this.beforeSave()
          this.save().then(res => {
            if (res && res.code == '000000') {
              this.$notify.success({
                title: "成功",
                message: "服务请求成功!"
              })
              this.exit()
            } else {
              this.$notify.error({
                title: "失败",
                message: "服务请求失败!"
              })
            }
          })
        }).catch(() => {
          this.$message({
            type: 'info',
            message: '已取消暂存'
          })
        })
      } else {
        this.$message.warning('暂存需要填写业务编号')
      }
    },

    showAml() {

    },

    exit: function () {
      this.$router.push('/business/home')
    }
  }
}